#!/usr/bin/env python
# -*- coding: iso-8859-15 -*-

# 2007 - Topazz <mfaya@free.fr>
# Version 1.3
# Usage : rm_special_char.py /absolute/subdir_path subdir_path -lower
# Add "-lower" option to convert upper cases to lower cases
# Recursively rename files and directories in a name compatible with MS-DOS FAT16 and URL (RFC 1738)
# All characters different from A–Z, 0–9 and $-_!'() are renamed

import os, sys, re, string, unicodedata

def process(current_item, format=""):
  try: os.chdir(current_item)
  except: return 1
  liste=os.listdir(".")
  for item in liste: rm_special_char(item, format)
  liste=os.listdir(".") # list again because names has changed
  for item in liste: process(item)
  os.chdir("..")

def rm_special_char(item, format):
  oldname=item
  newname = re.sub( r'%2f', '-', oldname) # / transformed by os.listdir in %2f
  newname = re.sub( r' ', '_', newname) # Space
  newname = re.sub( r'"', '\'\'', newname) # Double quote
  newname = re.sub( r'\x23', '-', newname) # # Hash; 'pound sign'
  newname = re.sub( r'\x25', '-', newname) # % Percent sign
  newname = re.sub( r'&', '-', newname) # Ampersand
  newname = re.sub( r'\x2A', '-', newname) # * Asterix
  newname = re.sub( r'\x2B', '-', newname) # + Plus sign
  newname = re.sub( r',', '-', newname) # Comma
  newname = re.sub( r'\x2F', '-', newname) # / Slash; forward slash
  newname = re.sub( r':', '-', newname) # Colon
  newname = re.sub( r';', '-', newname) # Semi-colon
  newname = re.sub( r'<', '-', newname) # Less-than sign
  newname = re.sub( r'=', '-', newname) # Equals
  newname = re.sub( r'>', '-', newname) # Greater-than sign
  newname = re.sub( r'\x3F', '-', newname) # Question mark
  newname = re.sub( r'@', '-', newname) # At sign; commercial at
  newname = re.sub( r'\x5B', '(', newname) # [ Open square bracket
  newname = re.sub( r'\x5C', '-', newname) # \ Backslash
  newname = re.sub( r'\x5D', ')', newname) # ] Close square bracket
  newname = re.sub( r'\x5E', '-', newname) # ^ Caret
  newname = re.sub( r'`', '\'', newname) # Grave accent
  newname = re.sub( r'\x7B', '(', newname) # { Left brace; left curly bracket
  newname = re.sub( r'\x7C', '-', newname) # | Vertical bar
  newname = re.sub( r'\x7D', ')', newname) # } Right brace; right curly bracket
  newname = re.sub( r'~', '-', newname) # Tilde
  newname = re.sub( r' ', '_', newname) # Non-breaking space
  newname = re.sub( r'¡', '!', newname) # Inverted exclamation mark
  newname = re.sub( r'¢', '-', newname) # Cent sign
  newname = re.sub( r'£', '-', newname) # Pound sign
  newname = re.sub( r'¤', '-', newname) # Currency sign
  newname = re.sub( r'¥', '-', newname) # Yen sign; yuan sign
  newname = re.sub( r'¦', '-', newname) # Broken bar; broken vertical bar
  newname = re.sub( r'§', '-', newname) # Section sign
  newname = re.sub( r'¨', '_', newname) # Diaeresis; spacing diaeresis
  newname = re.sub( r'©', '-', newname) # Copyright sign
  newname = re.sub( r'ª', '-', newname) # Feminine ordinal indicator
  newname = re.sub( r'«', '\'\'', newname) # Left-pointing double angle quotation mark; left pointing guillemet
  newname = re.sub( r'¬', '-', newname) # Not sign
  newname = re.sub( r'­', '-', newname) # Soft hyphen; discretionary hyphen
  newname = re.sub( r'®', '-', newname) # Registered sign; registered trade mark sign
  newname = re.sub( r'¯', '-', newname) # Macron; spacing macron; overline; APL overbar
  newname = re.sub( r'°', '-', newname) # Degree sign
  newname = re.sub( r'±', '-', newname) # Plus-minus sign; plus-or-minus sign
  newname = re.sub( r'²', '2', newname) # Superscript two; superscript digit two; squared
  newname = re.sub( r'³', '3', newname) # Superscript three; superscript digit three; cubed
  newname = re.sub( r'´', '\'', newname) # Acute accent; spacing acute
  newname = re.sub( r'µ', 'micro', newname) # Micro sign
  newname = re.sub( r'¶', '-', newname) # Pilcrow sign; paragraph sign
  newname = re.sub( r'·', '-', newname) # Middle dot; Georgian comma; Greek middle dot
  newname = re.sub( r'¸', '-', newname) # Cedilla; spacing cedilla
  newname = re.sub( r'¹', '1', newname) # Superscript one; superscript digit one
  newname = re.sub( r'º', '-', newname) # Masculine ordinal indicator
  newname = re.sub( r'»', '\'\'', newname) # Right-pointing double angle quotation mark; right pointing guillemet
  newname = re.sub( r'¼', '-', newname) # Vulgar fraction one quarter; fraction one quarter
  newname = re.sub( r'½', '-', newname) # Vulgar fraction one half; fraction one half
  newname = re.sub( r'¾', '-', newname) # Vulgar fraction three quarters; fraction 3 quarters
  newname = re.sub( r'¿', '-', newname) # Inverted question mark; turned question mark
  newname = re.sub( r'À', 'A', newname) # Latin capital letter A with grave; Latin capital letter A grave
  newname = re.sub( r'Á', 'A', newname) # Latin capital letter A with acute
  newname = re.sub( r'Â', 'A', newname) # Latin capital letter A with circumflex
  newname = re.sub( r'Ã', 'A', newname) # Latin capital letter A with tilde
  newname = re.sub( r'Ä', 'Ae', newname) # Latin capital letter A with diaeresis
  newname = re.sub( r'Å', 'A', newname) # Latin capital letter A with ring above; Latin capital letter A ring
  newname = re.sub( r'Æ', 'AE', newname) # Latin capital letter AE; Latin capital ligature AE
  newname = re.sub( r'Ç', 'C', newname) # Latin capital letter C with cedilla
  newname = re.sub( r'È', 'E', newname) # Latin capital letter E with grave
  newname = re.sub( r'É', 'E', newname) # Latin capital letter E with acute
  newname = re.sub( r'Ê', 'E', newname) # Latin capital letter E with circumflex
  newname = re.sub( r'Ë', 'E', newname) # Latin capital letter E with diaeresis
  newname = re.sub( r'Ì', 'I', newname) # Latin capital letter I with grave
  newname = re.sub( r'Í', 'I', newname) # Latin capital letter I with acute
  newname = re.sub( r'Î', 'I', newname) # Latin capital letter I with circumflex
  newname = re.sub( r'Ï', 'I', newname) # Latin capital letter I with diaeresis
  newname = re.sub( r'Ð', 'Dh', newname) # Latin capital letter ETH
  newname = re.sub( r'Ñ', 'N', newname) # Latin capital letter N with tilde
  newname = re.sub( r'Ò', 'O', newname) # Latin capital letter O with grave
  newname = re.sub( r'Ó', 'O', newname) # Latin capital letter O with acute
  newname = re.sub( r'Ô', 'O', newname) # Latin capital letter O with circumflex
  newname = re.sub( r'Õ', 'O', newname) # Latin capital letter O with tilde
  newname = re.sub( r'Ö', 'Oe', newname) # Latin capital letter O with diaeresis
  newname = re.sub( r'×', 'X', newname) # Multiplication sign
  newname = re.sub( r'Ø', 'Oe', newname) # Latin capital letter O with stroke; Latin capital letter O slash
  newname = re.sub( r'Ù', 'U', newname) # Latin capital letter U with grave
  newname = re.sub( r'Ú', 'U', newname) # Latin capital letter U with acute
  newname = re.sub( r'Û', 'U', newname) # Latin capital letter U with circumflex
  newname = re.sub( r'Ü', 'Ue', newname) # Latin capital letter U with diaeresis
  newname = re.sub( r'Ý', 'Y', newname) # Latin capital letter Y with acute
  newname = re.sub( r'Þ', 'Th', newname) # Latin capital letter THORN
  newname = re.sub( r'ß', 'ss', newname) # Latin small letter sharp s; ess-zed
  newname = re.sub( r'à', 'a', newname) # Latin small letter a with grave; Latin small letter a grave
  newname = re.sub( r'á', 'a', newname) # Latin small letter a with acute
  newname = re.sub( r'â', 'a', newname) # Latin small letter a with circumflex
  newname = re.sub( r'ã', 'a', newname) # Latin small letter a with tilde
  newname = re.sub( r'ä', 'ae', newname) # Latin small letter a with diaeresis
  newname = re.sub( r'å', 'a', newname) # Latin small letter a with ring above; Latin small letter a ring
  newname = re.sub( r'æ', 'ae', newname) # Latin small letter ae; Latin small ligature ae
  newname = re.sub( r'ç', 'c', newname) # Latin small letter c with cedilla
  newname = re.sub( r'è', 'e', newname) # Latin small letter e with grave
  newname = re.sub( r'é', 'e', newname) # Latin small letter e with acute
  newname = re.sub( r'ê', 'e', newname) # Latin small letter e with circumflex
  newname = re.sub( r'ë', 'e', newname) # Latin small letter e with diaeresis
  newname = re.sub( r'ì', 'i', newname) # Latin small letter i with grave
  newname = re.sub( r'í', 'i', newname) # Latin small letter i with acute
  newname = re.sub( r'î', 'i', newname) # Latin small letter i with circumflex
  newname = re.sub( r'ï', 'i', newname) # Latin small letter i with diaeresis
  newname = re.sub( r'ð', 'Dh', newname) # Latin small letter eth
  newname = re.sub( r'ñ', 'n', newname) # Latin small letter n with tilde
  newname = re.sub( r'ò', 'o', newname) # Latin small letter o with grave
  newname = re.sub( r'ó', 'o', newname) # Latin small letter o with acute
  newname = re.sub( r'ô', 'o', newname) # Latin small letter o with circumflex
  newname = re.sub( r'õ', 'o', newname) # Latin small letter o with tilde
  newname = re.sub( r'ö', 'oe', newname) # Latin small letter o with diaeresis
  newname = re.sub( r'÷', 'div', newname) # Division sign
  newname = re.sub( r'ø', 'oe', newname) # Latin small letter o with stroke; Latin small letter o slash
  newname = re.sub( r'ù', 'u', newname) # Latin small letter u with grave
  newname = re.sub( r'ú', 'u', newname) # Latin small letter u with acute
  newname = re.sub( r'û', 'u', newname) # Latin small letter u with circumflex
  newname = re.sub( r'ü', 'ue', newname) # Latin small letter u with diaeresis
  newname = re.sub( r'ý', 'y', newname) # Latin small letter y with acute
  newname = re.sub( r'þ', 'th', newname) # Latin small letter thorn
  newname = re.sub( r'ÿ', 'y', newname) # Latin small letter y with diaeresis  
  newname = newname.decode('iso8859-15').encode('ascii',"ignore")
  
  if format == '-lower': newname = string.lower(newname)

  if newname == '': newname = '-'
  
  if oldname != newname:
    while os.path.exists(newname): newname = '-' + newname
    print("Renaming \""+os.getcwd()+"/"+oldname+"\" -> \""+newname+"\"")
    os.rename(oldname,newname)

path = sys.argv[1]
dirname = sys.argv[2]
if len(sys.argv) >= 4: format = sys.argv[3]
else: format = "keep_upper"
process(path, format)
rm_special_char(dirname, format)